@@ -24,7 +24,7 @@
 #define FR_PROCESS_H
 
 #include <glib.h>
-#include <gtk/gtkobject.h>
+#include <gtk/gtk.h>
 #include <sys/types.h>
 #include "typedefs.h"
 
@@ -48,6 +48,8 @@ typedef struct {
 	GList      *raw;
 	LineFunc    line_func;
 	gpointer    line_data;
+	GIOStatus   status;
+	GError     *error;
 } FrChannelData;
 
 struct _FrProcess {
@@ -95,7 +97,7 @@ void        fr_process_add_arg              (FrProcess    *fr_proc,
 					     const char   *arg);
 void        fr_process_add_arg_concat       (FrProcess    *fr_proc,
 					     const char   *arg,
-					     ...);
+					     ...) G_GNUC_NULL_TERMINATED;
 void        fr_process_add_arg_printf       (FrProcess    *fr_proc,
 					     const char   *format,
 					     ...) G_GNUC_PRINTF (2, 3);
