@@ -35,6 +35,7 @@
 #include "fr-command-arj.h"
 #include "fr-command-cfile.h"
 #include "fr-command-cpio.h"
+#include "fr-command-dpkg.h"
 #include "fr-command-iso.h"
 #include "fr-command-jar.h"
 #include "fr-command-lha.h"
@@ -45,6 +46,7 @@
 #include "fr-command-zip.h"
 #include "fr-command-zoo.h"
 #include "fr-command-7z.h"
+#include "fr-command-lrzip.h"
 #include "fr-process.h"
 #include "fr-stock.h"
 #include "gconf-utils.h"
@@ -101,6 +103,8 @@ FrMimeTypeDescription mime_type_desc[] = {
 	{ "application/x-gzip",                 ".gz",       NULL, 0 },
 	{ "application/x-java-archive",         ".jar",      N_("Jar (.jar)"), 0 },
 	{ "application/x-lha",                  ".lzh",      N_("Lha (.lzh)"), 0 },
+	{ "application/x-lrzip",                ".lrz",      N_("Lrzip (.lrz)"), 0},
+	{ "application/x-lrzip-compressed-tar", ".tar.lrz",  N_("Tar compressed with lrzip (.tar.lrz)"), 0 },
 	{ "application/x-lzip",                 ".lz",       NULL, 0 },
 	{ "application/x-lzip-compressed-tar",  ".tar.lz",   N_("Tar compressed with lzip (.tar.lz)"), 0 },
 	{ "application/x-lzma",                 ".lzma",     NULL, 0 },
@@ -141,6 +145,7 @@ FrExtensionType file_ext_type[] = {
 	{ ".iso", "application/x-cd-image" },
 	{ ".jar", "application/x-java-archive" },
 	{ ".lha", "application/x-lha" },
+	{ ".lrz", "application/x-lrzip" },
 	{ ".lzh", "application/x-lha" },
 	{ ".lz", "application/x-lzip" },
 	{ ".lzma", "application/x-lzma" },
@@ -153,6 +158,7 @@ FrExtensionType file_ext_type[] = {
 	{ ".tar.bz", "application/x-bzip-compressed-tar" },
 	{ ".tar.bz2", "application/x-bzip-compressed-tar" },
 	{ ".tar.gz", "application/x-compressed-tar" },
+	{ ".tar.lrz", "application/x-lrzip-compressed-tar" },
 	{ ".tar.lz", "application/x-lzip-compressed-tar" },
 	{ ".tar.lzma", "application/x-lzma-compressed-tar" },
 	{ ".tar.lzo", "application/x-lzop-compressed-tar" },
@@ -548,11 +554,14 @@ fr_registered_command_get_potential_capabilities (FrRegisteredCommand *reg_com,
 {
 	int i;
 
+	if (mime_type == NULL)
+		return FR_COMMAND_CAN_DO_NOTHING;
+
 	for (i = 0; i < reg_com->caps->len; i++) {
 		FrMimeTypeCap *cap;
 
 		cap = g_ptr_array_index (reg_com->caps, i);
-		if (strcmp (mime_type, cap->mime_type) == 0)
+		if ((cap->mime_type != NULL) && (strcmp (mime_type, cap->mime_type) == 0))
 			return cap->potential_capabilities;
 	}
 
@@ -600,6 +609,7 @@ register_commands (void)
 	register_command (FR_TYPE_COMMAND_TAR);
 	register_command (FR_TYPE_COMMAND_CFILE);
 	register_command (FR_TYPE_COMMAND_7Z);
+	register_command (FR_TYPE_COMMAND_DPKG);
 
 	register_command (FR_TYPE_COMMAND_ACE);
 	register_command (FR_TYPE_COMMAND_ALZ);
@@ -613,6 +623,7 @@ register_commands (void)
 	register_command (FR_TYPE_COMMAND_RPM);
 	register_command (FR_TYPE_COMMAND_UNSTUFF);
 	register_command (FR_TYPE_COMMAND_ZIP);
+	register_command (FR_TYPE_COMMAND_LRZIP);
 	register_command (FR_TYPE_COMMAND_ZOO);
 }
 
@@ -669,6 +680,8 @@ update_registered_commands_capabilities (void)
 {
 	int i;
 
+	g_hash_table_remove_all (ProgramsCache);
+
 	for (i = 0; i < Registered_Commands->len; i++) {
 		FrRegisteredCommand *reg_com;
 		FrCommand           *command;
